<?php
/**
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.4.0.1
 * @package OpenVCP-v0.4
 */

require_once('AdminFactory.class.php');
require_once("CustomerFactory.class.php");
require_once("NodeFactory.class.php");

class Vserver {

	private $vserverinfo = null;
	private $userinfo = null;
	private $DB = null;
	private $XML = null;
	private $SOCKET = null;
	private $AUTH = null;
	private $node = null;
	private $customer = null;
	private $admin = null;
	private $nodeFactory = null;

	function __construct($vserver) {
		$this->SOCKET =& OpenVCP::call('Socket', 'getReference');
		$this->XML    =& OpenVCP::call('XML', 'getReference');
		$this->DB    =& OpenVCP::call('DB', 'getReference');
		$this->AUTH   =& OpenVCP::call('Auth', 'getReference');
		$this->userinfo = $this->AUTH->user_session;
		$this->vserverinfo = $vserver;
		$this->nodeFactory = NodeFactory::getNodeFactory();
		$this->admin = AdminFactory::getInstance()->getUserByID($this->vserverinfo['adminid']);
	}

	public function disableVServer() {
		$this->setAutostart(0);
		$this->DB->update($this->DB->table_vservers, array('disable' => 'y'), array('id' => $this->getId()));
		$state = $this->getFullState();
		if($state[0] == "Online")
			$this->controlStop();
	}
	
	public function enableVserver() {
		$this->setAutostart(1);
		$this->DB->update($this->DB->table_vservers, array('disable' => 'n'), array('id' => $this->getId()));
		$state = $this->getFullState();
		if($state[0] == "Offline")
			$this->controlStart();
	}
	
	public function isDisabled($intern = false) {
		if($intern) {
			if($this->vserverinfo['disable'] == 'y' && $this->userinfo['is_admin'] == 0)
				return true;
			else
				return false;
		}
		else {
			if($this->vserverinfo['disable'] == 'y')
				return true;
			else
				return false;
		}
	}
	
	/**
	 * returns the id
	 */
	public function getId()
	{
		return $this->vserverinfo['id'];
	}

	/**
	 * returns the adminid
	 *
	 * @return int
	 */
	public function getAdmin()
	{
		return $this->vserverinfo['adminid'];
	}

	/**
	 * returns the userid
	 *
	 * @return int
	 */
	public function getUser()
	{
		return $this->vserverinfo['userid'];
	}

	/**
	 * returns the name of the vserver
	 *
	 * @return string
	 */
	public function getName()
	{
		return $this->vserverinfo['name'];
	}

	/**
	 * returns the name of the node
	 *
	 * @return String node
	 */
	public function getNode()
	{
		return $this->nodeFactory->getNodeById($this->vserverinfo['nodeid'])->getName();
	}

	/**
	 * returns an object of Admin which is the admin of the Vserver
	 *
	 * @return Admin
	 */
	public function getAdminObject()
	{
		return $this->admin;
	}

	/**
	 * returns an object of Customer which is the customer of the Vserver
	 *
	 * @return Customer
	 */
	public function getUserObject()
	{
		return $this->customer = CustomerFactory::getInstance()->getUserByID($this->vserverinfo['userid']);
	}

	/**
	 * returns an object of Node which is the node of the Vserver
	 *
	 * @return Node
	 */
	public function getNodeObject()
	{
		return $this->nodeFactory->getNodeById($this->vserverinfo['nodeid']);
	}

	public function updateInfo($userid, $adminid) {
		$this->DB->update($this->DB->table_vservers, array('userid' => $userid, 'adminid' => $adminid), array('id' => $this->vserverinfo['id']));
		$this->vserverinfo['userid'] = $userid;
		$this->vserverinfo['adminid'] = $adminid;
		$this->admin = AdminFactory::getInstance()->getUserByID($this->vserverinfo['adminid']);
	}
	
	/**
	 * returns the hostname of the vserver
	 */
	public function getHostname()
	{
		$request = $this->XML->newRequestObject("vserver.get_hostname");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");
		return $values[0];
	}

	/**
	 * Sets the vservers hostname
	 *
	 * @param string $hostname
	 */
	public function setHostname($hostname)
	{
		if($this->isDisabled(true))
			return "error vserver is disabled";
		$request = $this->XML->newRequestObject("vserver.set_hostname");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $hostname);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * returns the ip's of the vserver
	 *
	 * @return XMLObject responseid
	 */
	public function getIps()
	{
		$request = $this->XML->newRequestObject("vserver.get_ips");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		return $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	public function addIp($ip, $dev, $mask, $alias, $bcast, $nodev="", $tuntap="")
	{
		if(strlen($alias) > 10) {
			$alias = substr($alias, -10, 10);
		}
		$request = $this->XML->newRequestObject("vserver.add_iface");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addStructMember($request, "ip", "string", $ip);
		$this->XML->addStructMember($request, "dev", "string", $dev);
		if($mask != "") $this->XML->addStructMember($request, "mask", "string", $mask);
		if($alias != "") $this->XML->addStructMember($request, "name", "string", $alias);
		if($bcast != "") $this->XML->addStructMember($request, "bcast", "string", $bcast);
		if($nodev != "") $this->XML->addStructMember($request, "nodev", "string", "nodev");
		if($tuntap == "tun") $this->XML->addStructMember($request, "tun", "string", "tun");
		if($tuntap == "tap") $this->XML->addStructMember($request, "tap", "string", "tap");
		
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$this->DB->update($this->DB->table_network, array('vserverid' => $this->vserverinfo['id'], 'used' => 'y'), array('id' => $ip));
	}
	
	/**
	 * adds a interface directly from the db
	 */
	public function addIpByDB($id, $dev, $alias) {
		$ipInfo = $this->DB->select($this->DB->table_network, NULL, NULL, array('id' => $id));
		$ipInfo = $ipInfo[0];
		$this->addIp($ipInfo['ip'], $dev, $ipInfo['mask'], $alias, $ipInfo['bcast'], ($dev == 'nodev' ? 'nodev' : ''));
		$this->DB->update($this->DB->table_network, array('vserverid' => $this->vserverinfo['id'], 'used' => 'y'), array('id' => $id));
		
	}

	public function removeIp($id, $ip)
	{
		$request = $this->XML->newRequestObject("vserver.remove_iface");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $id);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$this->DB->update($this->DB->table_network, array('vserverid' => '0', 'used' => 'n'), array('ip' => $ip));
	}

	/**
	 * returns the traffic of the
	 * given month and year
	 *
	 * @param int $month
	 * @param int $year
	 */
	public function getTrafficTotal($month, $year)
	{
		$request = $this->XML->newRequestObject("vserver.traffic_total");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "int", $month);
		$this->XML->addValue($request, "int", $year);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $this->XML->getNextStruct($response);
	}

	/**
	 * returns the current state
	 *
	 * @return string state
	 * @deprecated 
	 */
	public function getState()
	{
		$request = $this->XML->newRequestObject("vserver.get_state");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");
		return $values[0];
	}

	/**
	 * returns the current state
	 *
	 * @return string state
	 */
	public function getFullState()
	{
		$request = $this->XML->newRequestObject("vserver.get_state");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");
		return $values;
	}
	
	/**
	 * returns the current load
	 *
	 * @param boolean $format
	 * @return string load
	 */
	public function getLoad($format = false)
	{
		$request = $this->XML->newRequestObject("vserver.get_load");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");

		if($format)
		{
			$data = explode(" ", $values[0]);
			$returndata = "";
			for($a = 0; $a < 3; $a++)
			{
				if($data[$a] < 1) $returndata .= "<font color=\"green\">".$data[$a]."</font>";
				else if($data[$a] < 5) $returndata .= "<font color=\"#FFCC00\">".$data[$a]."</font>";
				else $returndata .= "<font color=\"red\">".$data[$a]."</font>";
				($a < 2 && $data[$a] != "") ? $returndata .= "|" : $returndata .= "";
			}
			return $returndata;
		}
		else
		return $values[0];
	}

	/**
	 * returns the crrent uptime
	 *
	 * @param boolean $format
	 * @return string uptime
	 */
	public function getUptime($format)
	{
		$request = $this->XML->newRequestObject("node.get_uptime");
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$uptime = $this->XML->getValues($response, "string");
		$request = $this->XML->newRequestObject("vserver.get_uptime");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$biasuptime = $this->XML->getValues($response, "string");
		$biasuptime = $biasuptime[0];
		if($format && $biasuptime != "")
		{
			$input = explode(" ", $uptime[0]);
			$biasuptime == "0 0" ? $input[0] = 0 : $biasuptime = $biasuptime;
			$input[0] = $input[0] - $biasuptime;
			if($input[0] != 0)
			{
				$reference = 3600 * 24;
				$days = 0;
				while($input[0] > ($reference * $days))
				{
					$days++;
				}
				$days--;
				$input[0] = $input[0] - ($days * $reference);
				$hours = 0;
				while($input[0] > (3600 * $hours))
				{
					$hours++;
				}
				$hours--;
				$input[0] = $input[0] - ($hours * 3600);
				$mins = 0;
				while($input[0] > ($mins * 60))
				{
					$mins++;
				}
				$mins = ($mins < 10) ? "0".$mins : $mins;
				$hours = ($hours < 10) ? "0".$hours : $hours;
			}
			else
			{
				$days = 0;
				$hours = "00";
				$mins = "00";
			}
			return $days." "._("days").", ".$hours.":".$mins;
		}
		else if($biasuptime == "") {
			return "";
		}
		else
		return $biasuptime;
	}

	/**
	 * returns the current filters set for the
	 * vserver
	 *
	 * @return XMLObject
	 *
	 */
	public function getFilter()
	{
		$request = $this->XML->newRequestObject("vserver.get_filter");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		return $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * adds a new Rule to the vserver
	 *
	 * @param unknown_type $direction
	 * @param unknown_type $proto
	 * @param unknown_type $srcip
	 * @param unknown_type $srcport
	 * @param unknown_type $destip
	 * @param unknown_type $destport
	 * @param unknown_type $target
	 * @param unknown_type $match
	 * @param unknown_type $matchValue
	 */
	public function setFilter($direction, $proto, $srcip, $srcport, $destip, $destport, $target, $match, $matchValue)
	{
		$request = $this->XML->newRequestObject("vserver.set_filter");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addStructMember($request, "direction", "string", $direction);
		$this->XML->addStructMember($request, "proto", "string", $proto);
		$this->XML->addStructMember($request, "srcip", "string", $srcip);
		$this->XML->addStructMember($request, "srcport", "string", $srcport);
		$this->XML->addStructMember($request, "destip", "string", $destip);
		$this->XML->addStructMember($request, "destport", "string", $destport);
		$this->XML->addStructMember($request, "target", "string", $target);
		$this->XML->addStructMember($request, "match", "string", $match);
		$this->XML->addStructMember($request, "match-value", "string", $matchValue);
		$this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * Deletes a rule of the vserver
	 * specified by $id
	 *
	 * @param int $id
	 */
	public function delFilter($id)
	{
		$request = $this->XML->newRequestObject("vserver.del_filter");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $id);
		$this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * starts a vserver
	 */
	public function controlStart()
	{
		if($this->isDisabled(true))
			return "error vserver is disabled";
		$request = $this->XML->newRequestObject("vserver.start");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$dmesg = $this->XML->getValues($response, "string");
		return $this->filterDmesg($dmesg);
	}

	/**
	 * stops a vserver
	 */
	public function controlStop()
	{
		if($this->isDisabled(true))
			return "error vserver is disabled";
		$request = $this->XML->newRequestObject("vserver.stop");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$dmesg = $this->XML->getValues($response, "string");
		return $this->filterDmesg($dmesg);
	}

	/**
	 * removes a vserver
	 *
	 * @return boolean success
	 */
	public function controlRemove()
	{
		$request = $this->XML->newRequestObject("vserver.remove");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$result = $this->XML->getValues($response, "boolean");
		return $result[0];
	}

	/**
	 * gets the flag of autostart
	 *
	 * @return boolean autostart
	 */
	public function getAutostart() {
		$request = $this->XML->newRequestObject("vserver.get_mark");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$autostart = $this->XML->getValues($response, "boolean");
		return $autostart[0];
	}

	/**
	 * changes the autostart
	 *
	 * @param boolean $mark
	 */
	public function setAutostart($mark) {
		$request = $this->XML->newRequestObject("vserver.set_mark");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "boolean", $mark);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * sets the schedule settings to a vserver
	 * 
	 * @param int $schedule
	 * @param int $interval
	 * @param int $prioritybias
	 * @param int $tokens
	 * @param int $tokensmin
	 * @param int $tokensmax
	 * 
	 * @return boolean
	 */
	public function setSchedule($schedule, $interval = 32, $prioritybias = 0, $tokens = 500, $tokensmin = 200, $tokensmax = 1000, $fill_rate2 = 1, $interval2 = 5, $idle_time = false) {
		$request = $this->XML->newRequestObject("vserver.set_schedule");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addStructMember($request, "fill-rate", "string", number_format($schedule, 0));
		$this->XML->addStructMember($request, "interval", "string", $interval);
		$this->XML->addStructMember($request, "priority-bias", "string", $prioritybias);
		$this->XML->addStructMember($request, "tokens", "string", $tokens);
		$this->XML->addStructMember($request, "tokens-min", "string", $tokensmin);
		$this->XML->addStructMember($request, "tokens-max", "string", $tokensmax);
		$this->XML->addStructMember($request, "fill-rate2", "string", $fill_rate2);
		$this->XML->addStructMember($request, "interval2", "string", $interval2);
		if($idle_time == 0) {
			$this->XML->addStructMember($request, "idle-time", "string", "");
		}
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		if($this->XML->getValues($response, 'boolean')) {
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * returns current schedule settings
	 */
	public function getSchedule() {
		$request = $this->XML->newRequestObject("vserver.get_schedule");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $this->XML->getNextStruct($response);
	}
	
	/**
	 * sets a new rlimit
	 * 
	 * @param string $type
	 * @param int $value
	 * @return boolean
	 */
	public function setLimit($type, $value) {
		$request = $this->XML->newRequestObject("vserver.set_rlimit");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $type);
		$this->XML->addValue($request, "string", $value);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		if($this->XML->getValues($response, 'boolean')) {
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * returns the current set rlimits
	 * 
	 * @return XML
	 */
	public function getLimit() {
		$request = $this->XML->newRequestObject("vserver.get_rlimit");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		return $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}
	
	/**
	 * gets all flag stats
	 *
	 * @return array Flags
	 */
	public function getFlags()
	{
		$request = $this->XML->newRequestObject("vserver.get_flags");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $this->XML->getNextArray($response);
	}

	/**
	 * add flag
	 * @param String $flag
	 */
	public function addFlag($flag)
	{
		$request = $this->XML->newRequestObject("vserver.add_flag");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $flag);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * Sets the Bandwidth of a vserver
	 *
	 * @param string $direction
	 * @param int $rate in kbyte/s
	 * @param int $ceil in kbyte/s
	 * @param int $burst in byte
	 * @param int $cbust in byte
	 */
	public function setBandwidth($direction, $rate, $ceil, $burst = 0, $cbust = 0)
	{
		$request = $this->XML->newRequestObject("vserver.set_bandwidth");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addStructMember($request, "direction", "string", $direction);
		$this->XML->addStructMember($request, "rate", "int", $rate);
		$this->XML->addStructMember($request, "ceil", "int", $ceil);
		$this->XML->addStructMember($request, "burst", "int", $burst);
		$this->XML->addStructMember($request, "cburst", "int", $cbust);
		$this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}
	
	/**
	 * Deletes a Bandwidth limitation
	 *
	 * @param string $direction
	 */
	public function delBandwidth($direction)
	{
		$request = $this->XML->newRequestObject("vserver.del_bandwidth");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $direction);
		$this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * returns the current bandwidth settings of a vserver
	 *
	 * @return XML Object
	 */
	public function getBandwidth()
	{
		$request = $this->XML->newRequestObject("vserver.get_bandwidth");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		return $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	public function setTrafficLimit($limit, $mail, $type)
	{
		$request = $this->XML->newRequestObject("vserver.set_trafficlimit");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "int", $limit);
		$this->XML->addValue($request, "string", $mail);
		$this->XML->addValue($request, "string", $type);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$returnValue = $this->XML->getValues($response, "boolean");
		return $returnValue[0];
	}
	
	public function setSpace($spacetotal, $inodestotal, $reserved) {
		$request = $this->XML->newRequestObject("vserver.set_space");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addStructMember($request, "directory", "string", "/");
		$this->XML->addStructMember($request, "space_total", "string", $spacetotal);
		$this->XML->addStructMember($request, "inodes_total", "string", $inodestotal);
		$this->XML->addStructMember($request, "reserved", "string", $reserved);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}
	
	/**
	 * gets the current space settings of the vserver
	 * 
	 * @return array space
	 */
	public function getSpace() {
		$request = $this->XML->newRequestObject("vserver.get_space");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $this->XML->getNextStruct($response);
	}
	
	/**
	 * returns the current settings of the hard traffic limits
	 *
	 * @return array(limit, email)
	 */
	public function getTrafficLimit()
	{
		$request = $this->XML->newRequestObject("vserver.get_trafficlimit");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $response;
	}
	
	public function delTrafficLimit($type)
	{
		$request = $this->XML->newRequestObject("vserver.del_trafficlimit");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $type);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$returnValue = $this->XML->getValues($response, "boolean");
		return $returnValue[0];
	}
	
	/**
	 * delete a flag
	 * @param String $flag
	 */
	public function removeFlag($flag) {
		$request = $this->XML->newRequestObject("vserver.remove_flag");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $flag);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * Installs in the vserver the image
	 * specified with the imagename
	 *
	 * @param string $imagename
	 * @return boolean
	 */
	public function controlSetupImage($imagename) {
		if($this->isDisabled(true))
			return "error vserver is disabled";
		$hostname = $this->getHostname();
		$request = $this->XML->newRequestObject("setup.vserver_image");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $imagename);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$reinstallstate = $this->XML->getValues($response, "boolean");
		$this->setHostname($hostname);
		return $reinstallstate[0];
	}

	/**
	 * changes the vservers root password
	 *
	 * @param string $password
	 */
	public function controlSetupPassword($password) {
		if($this->isDisabled(true))
			return "error vserver is disabled";
		$request = $this->XML->newRequestObject("vserver.set_rootpw");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $password);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$returnvalue = $this->XML->getValues($response, 'boolean');
		return $returnvalue[0];
	}

	/**
	 * get the traffic statistics of a year
	 *
	 * @param int $year
	 */
	public function getTrafficYear($year) {
		$traffic = array();
		for($i = 1; $i <= 12; $i++)
		{
			$request = $this->XML->newRequestObject("vserver.traffic_total");
			$this->XML->addValue($request, "string", $this->vserverinfo['name']);
			$this->XML->addValue($request, "int", $i);
			$this->XML->addValue($request, "int", $year);
			$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
			$trafficRaw = $this->XML->getNextStruct($response);
			$traffic[$i]['in'] = $trafficRaw['in'];
			$traffic[$i]['out'] = $trafficRaw['out'];
			$traffic[$i]['total'] = $trafficRaw['total'];
		}
		return $traffic;
	}

	/**
	 * get the traffic statistics of a month
	 *
	 * @param int $month
	 * @param int $year
	 */
	public function getTrafficMonth($month, $year) {
		$request = $this->XML->newRequestObject("vserver.traffic");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "int", $month);
		$this->XML->addValue($request, "int", $year);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$traffic = array();
		while(($element = $this->XML->getNextStruct($response)) != NULL)
		{
			$traffic[$element['day']][$element['ip']]['in'] = $element['in'];
			$traffic[$element['day']][$element['ip']]['out'] = $element['out'];
			$traffic[$element['day']][$element['ip']]['total'] = $element['total'];
		}
		return $traffic;
	}

	/**
	 * get the traffic of a day
	 *
	 * @param int $day
	 * @param int $month
	 * @param int $year
	 */
	public function getTrafficDay($day, $month, $year) {
		$request = $this->XML->newRequestObject("vserver.traffic_hour");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "int", $month);
		$this->XML->addValue($request, "int", $year);
		$this->XML->addValue($request, "int", $day);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$traffic = array();
		while(($element = $this->XML->getNextStruct($response)) != NULL)
		{
			$traffic[$element['hour']][$element['ip']]['in'] = $element['in'];
			$traffic[$element['hour']][$element['ip']]['out'] = $element['out'];
			$traffic[$element['hour']][$element['ip']]['total'] = $element['total'];
		}
		return $traffic;
	}

	public function setRescue() {
		if($this->isDisabled(true))
			return "error vserver is disabled";
		$request = $this->XML->newRequestObject("vserver.rescue");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $this->XML->getValues($response, "string");
	}
	
	/**
	 * get the resources statistics
	 *
	 * @param int $day
	 * @param int $month
	 * @param int $year
	 */
	public function getResourcesHour($day, $month, $year) {
		$request = $this->XML->newRequestObject("vserver.resources_hour");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "int", $month);
		$this->XML->addValue($request, "int", $year);
		$this->XML->addValue($request, "int", $day);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$resources = array();
		while(($element = $this->XML->getNextStruct($response)) != NULL)
		{
			foreach($element as $key => $value)
			{
				if($key != 'hour')
				$resources[$element['hour']][$key] = $element[$key];
			}
		}
		return $resources;
	}

	private function filterDmesg($dmesg) {
		$filterdMessages = array();
		foreach($dmesg as $key => $value) {
			$lines = explode('<br />', nl2br($value));
			$msg = null;
			foreach($lines as $line) {
				if(!is_string(strstr($line, 'stty')) && !is_string(strstr($line, "util-vserver")))
					$msg[] = $line."<br />";
			}
			$filterdMessages[$key] = implode('', $msg);
		}
		return $filterdMessages;
	}
	
	/**
	 * creates a new VServer and returns the newly created vserver
	 * 
	 * @param string $vservername
	 * @param Node $node
	 * @param Admin $admin
	 * @param Customer $customer
	 */
	static public function createNewVServer($vservername, Node $node, Admin $admin, Customer $customer) {
		$XML = OpenVCP::call('XML', 'getReference');
		$SOCKET = OpenVCP::call('Socket', 'getReference');
		$DB = OpenVCP::call('DB', 'getReference');
		
		$request = $XML->newRequestObject("setup.vserver_config");
		$XML->addValue($request, "string", $vservername);
		$response = $SOCKET->socket_exec($node->getId(), $request);

		//get new XID of the vServer
		$request = $XML->newRequestObject("vserver.get_xid");
		$XML->addValue($request, "string", $vservername);
		$response = $SOCKET->socket_exec($node->getId(), $request);
		$xid = $XML->getValues($response, "int");

		//inserting vServer into DB
		$vsId = $DB->insert($DB->table_vservers, array('', $vservername, $xid[0], $node->getId(), $admin->getId(), $customer->getId(), 'n'));
		if($admin->getSetting("vserver_prefix").$admin->getSetting("vserver_prefix_last") == $vservername)
		{
			$oldValue = $admin->getSetting("vserver_prefix_last");
			$oldValue += 1;
			$admin->setSetting("vserver_prefix_last", $oldValue);
		}
		$vserver = VserverFactory::getInstance()->getVServerByNameAndNode($vservername, $node->getId());
		return $vserver;
	}
	
}
?>
